#!/usr/bin/env python
# -*- coding: utf-8 -*-
import cv2
import numpy as np

## @brief Represents an image and provides functionalities for masking and displaying it.
class Image(object):

    ## @brief Initializes the Image object by loading an image from the given file path.
    # @param imageFilePath Path to the image file to be loaded.
    def __init__(self, imageFilePath):
        self.__data: np.ndarray = cv2.imread(imageFilePath)

    ## @brief Applies a mask to the image based on the given side and projector parameters.
    # @param side The side of the projection to apply the mask ("left" or "right").
    # @param projectionWidthCm The width of the projection in centimeters.
    # @param distanceBetweenProjectorCm The distance between the projectors in centimeters.
    # @param gammaValue The gamma value to apply to the mask.
    def mask(self, side, projectionWidthCm, distanceBetweenProjectorCm, gammaValue):
        rows = self.__data.shape[0]
        columns = self.__data.shape[1]

        # Calculate the overlap ratio and the number of pixels affected by the overlap
        overlap = (projectionWidthCm - 2 * distanceBetweenProjectorCm) / (
                projectionWidthCm - distanceBetweenProjectorCm)
        result_pixel_overlap = int(overlap * columns)

        # Create the mask based on the side
        mask = np.ones(columns)
        if side == "right":
            mask[:result_pixel_overlap] = np.linspace(0, 1, result_pixel_overlap) ** gammaValue
        elif side == "left":
            mask[-result_pixel_overlap:] = np.linspace(1, 0, result_pixel_overlap) ** gammaValue

        # Apply the mask to the image
        mask = np.stack([mask] * rows)
        mask = np.stack([mask] * 3, axis=-1)

        self.__data = self.__data * mask

        # Clip the values to the valid range [0, 255] and convert back to uint8
        self.__data = np.clip(self.__data, 0, 255).astype(np.uint8)

    ## @brief Displays the image in a window.
    # The window will remain open until any key is pressed.
    def displayImage(self):
        cv2.imshow('Loaded Image', self.__data)
        cv2.waitKey(0)
        cv2.destroyAllWindows()