#!/usr/bin/env python
# -*- coding: utf-8 -*-
##
# @file image_configuration.py
# @brief Provides an ImageConfiguration class for reading and parsing image-related configuration from environment variables.
#
# This script uses `python-dotenv` to load environment variables from a specified
# file and provides methods to retrieve commonly used image-projection settings such as:
# - Image file path
# - Side (left or right)
# - Distance between projectors
# - Projection width
# - Gamma value
#
# Usage Example:
# @code{.py}
#   from image_configuration import ImageConfiguration
#   config = ImageConfiguration(".env")
#   image_path = config.getImageFilePath()
#   side = config.getSide()
#   distance_cm = config.getDistanceBetweenProjectorCm()
#   projection_width_cm = config.getProjectionWidthCm()
#   gamma = config.getGammaValue()
# @endcode

import os
import sys

# Insert the 'lib' directory to the system path so we can import from there.
sys.path.insert(0, os.path.join(os.path.dirname(__file__), 'lib'))

from dotenv import load_dotenv


##
# @class ImageConfiguration
# @brief A class for managing image-related configuration settings stored in environment variables.
#
# This class is responsible for loading environment variables from a configuration file
# (using `load_dotenv`) and providing convenient getters for the necessary values.
class ImageConfiguration(object):

    ##
    # @brief Constructor to initialize the ImageConfiguration object.
    # @param configFilePath A string specifying the path to the dotenv configuration file.
    #
    # The constructor loads environment variables from the specified file, making
    # them available via the getter methods.
    def __init__(self, configFilePath):
        self.configFilePath = configFilePath
        load_dotenv(configFilePath)

    ##
    # @brief Retrieve the path of the image file from environment variables.
    # @return A string representing the file path of the image.
    def getImageFilePath(self) -> str:
        return os.getenv("IMAGE_FILE_PATH")

    ##
    # @brief Retrieve the 'SIDE' setting (e.g., "left" or "right").
    # @return A string representing the side setting.
    def getSide(self) -> str:
        return os.getenv("SIDE")

    ##
    # @brief Retrieve the distance between projectors in centimeters.
    # @return An integer specifying the distance between projectors.
    def getDistanceBetweenProjectorCm(self) -> int:
        return int(os.getenv("DISTANCE_BETWEEN_PROJECTOR_CM"))

    ##
    # @brief Retrieve the projection width in centimeters.
    # @return An integer specifying the width of the projection.
    def getProjectionWidthCm(self) -> int:
        return int(os.getenv("PROJECTION_WIDTH_CM"))

    ##
    # @brief Retrieve the gamma value for image display correction.
    # @return A float representing the gamma correction value.
    def getGammaValue(self) -> float:
        return float(os.getenv("GAMMA_VALUE"))
